@@ -50,30 +50,45 @@ namespace FileOperations
 static void copyMoveOrLink(Operation operation, const QList<QUrl>& urlList, QWidget* parent, ContextManager* contextManager)
 {
     Q_ASSERT(!urlList.isEmpty());
+    const int numberOfImages = urlList.count();
 
     QFileDialog dialog(parent->nativeParentWidget(), QString());
     dialog.setAcceptMode(QFileDialog::AcceptSave);
 
+    // Figure out what the window title and buttons should say,
+    // depending on the operation and how many images are selected
     switch (operation) {
     case COPY:
-        dialog.setWindowTitle(i18nc("@title:window", "Copy To"));
+        if (numberOfImages == 1) {
+            dialog.setWindowTitle(i18nc("@title:window %1 file name", "Copy %1", urlList.constFirst().fileName()));
+        } else {
+            dialog.setWindowTitle(i18ncp("@title:window %1 number of images", "Copy %1 image", "Copy %1 images", numberOfImages));
+        }
         dialog.setLabelText(QFileDialog::DialogLabel::Accept, i18nc("@action:button", "Copy"));
         break;
     case MOVE:
-        dialog.setWindowTitle(i18nc("@title:window", "Move To"));
+        if (numberOfImages == 1) {
+            dialog.setWindowTitle(i18nc("@title:window %1 file name", "Move %1", urlList.constFirst().fileName()));
+        } else {
+            dialog.setWindowTitle(i18ncp("@title:window %1 number of images", "Move %1 image", "Move %1 images", numberOfImages));
+        }
         dialog.setLabelText(QFileDialog::DialogLabel::Accept, i18nc("@action:button", "Move"));
         break;
     case LINK:
-        dialog.setWindowTitle(i18nc("@title:window", "Link To"));
+        if (numberOfImages == 1) {
+            dialog.setWindowTitle(i18nc("@title:window %1 file name", "Link %1", urlList.constFirst().fileName()));
+        } else {
+            dialog.setWindowTitle(i18ncp("@title:window %1 number of images", "Link %1 image", "Link %1 images", numberOfImages));
+        }
         dialog.setLabelText(QFileDialog::DialogLabel::Accept, i18nc("@action:button", "Link"));
         break;
     default:
         Q_ASSERT(0);
     }
 
-    if (urlList.count() == 1) {
+    if (numberOfImages == 1) {
         dialog.setFileMode(QFileDialog::AnyFile);
-        dialog.selectUrl(urlList.first());
+        dialog.selectUrl(urlList.constFirst());
     } else {
         dialog.setFileMode(QFileDialog::Directory);
         dialog.setOption(QFileDialog::ShowDirsOnly, true);
@@ -102,7 +117,7 @@ static void copyMoveOrLink(Operation operation, const QList<QUrl>& urlList, QWid
         Q_ASSERT(0);
     }
     KJobWidgets::setWindow(job, parent);
-    job->ui()->setAutoErrorHandlingEnabled(true);
+    job->uiDelegate()->setAutoErrorHandlingEnabled(true);
 }
 
 static void delOrTrash(KIO::JobUiDelegate::DeletionType deletionType, const QList<QUrl>& urlList, QWidget* parent)
@@ -218,7 +233,7 @@ void rename(const QUrl &oldUrl, QWidget* parent)
     KIO::SimpleJob* job = KIO::rename(oldUrl, newUrl, KIO::HideProgressInfo);
     KJobWidgets::setWindow(job, parent);
     if (!job->exec()) {
-        job->ui()->showErrorMessage();
+        job->uiDelegate()->showErrorMessage();
         return;
     }
     ThumbnailProvider::moveThumbnail(oldUrl, newUrl);
