@@ -49,6 +49,7 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 #include <KUrlMimeData>
 #include <KFileItemActions>
 #include <KFileItemListProperties>
+#include <KIO/OpenFileManagerWindowJob>
 
 // Local
 #include <lib/contextmanager.h>
@@ -63,17 +64,7 @@ namespace Gwenview
 
 QList<QUrl> FileOpsContextManagerItem::urlList() const
 {
-    QList<QUrl> urlList;
-
-    KFileItemList list = contextManager()->selectedFileItemList();
-    if (list.count() > 0) {
-        urlList = list.urlList();
-    } else {
-        QUrl url = contextManager()->currentUrl();
-        Q_ASSERT(url.isValid());
-        urlList << url;
-    }
-    return urlList;
+    return contextManager()->selectedFileItemList().urlList();
 }
 
 void FileOpsContextManagerItem::updateServiceList()
@@ -187,6 +178,9 @@ FileOpsContextManagerItem::FileOpsContextManagerItem(ContextManager* manager, QL
     connect(menu, &QMenu::aboutToShow, this, &FileOpsContextManagerItem::populateOpenMenu);
     connect(menu, &QMenu::triggered, this, &FileOpsContextManagerItem::openWith);
 
+    mOpenContainingFolderAction = file->addAction("file_open_containing_folder", this, SLOT(openContainingFolder()));
+    mOpenContainingFolderAction->setText(i18n("Open Containing Folder"));
+
     mRegularFileActionList
             << mRenameAction
             << mTrashAction
@@ -197,6 +191,7 @@ FileOpsContextManagerItem::FileOpsContextManagerItem(ContextManager* manager, QL
             << mLinkToAction
             << createSeparator(this)
             << mOpenWithAction
+            << mOpenContainingFolderAction
             << mShowPropertiesAction
             << createSeparator(this)
             << mCreateFolderAction
@@ -243,6 +238,7 @@ void FileOpsContextManagerItem::updateActions()
     mDelAction->setEnabled(selectionNotEmpty);
     mOpenWithAction->setEnabled(selectionNotEmpty);
     mRenameAction->setEnabled(count == 1);
+    mOpenContainingFolderAction->setEnabled(selectionNotEmpty);
 
     mCreateFolderAction->setEnabled(dirUrlIsValid);
     mShowPropertiesAction->setEnabled(dirUrlIsValid || urlIsValid);
@@ -329,17 +325,17 @@ void FileOpsContextManagerItem::restore()
 
 void FileOpsContextManagerItem::copyTo()
 {
-    FileOperations::copyTo(urlList(), mGroup);
+    FileOperations::copyTo(urlList(), widget(), contextManager());
 }
 
 void FileOpsContextManagerItem::moveTo()
 {
-    FileOperations::moveTo(urlList(), mGroup);
+    FileOperations::moveTo(urlList(), widget(), contextManager());
 }
 
 void FileOpsContextManagerItem::linkTo()
 {
-    FileOperations::linkTo(urlList(), mGroup);
+    FileOperations::linkTo(urlList(), widget(), contextManager());
 }
 
 void FileOpsContextManagerItem::rename()
@@ -412,4 +408,9 @@ void FileOpsContextManagerItem::openWith(QAction* action)
     KRun::runService(*service, list, mGroup);
 }
 
+void FileOpsContextManagerItem::openContainingFolder()
+{
+    KIO::highlightInFileManager(urlList());
+}
+
 } // namespace
