@@ -30,8 +30,8 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 #include <QIcon>
 
 // KDE
-#include <KGlobalSettings>
 #include <KIconLoader>
+#include <QFontDatabase>
 
 // Local
 
@@ -45,13 +45,13 @@ namespace Gwenview
 class SideBarButton : public QToolButton
 {
 protected:
-    virtual void paintEvent(QPaintEvent* event)
+    void paintEvent(QPaintEvent* event) Q_DECL_OVERRIDE
     {
         forceIcon();
         QToolButton::paintEvent(event);
     }
 
-    virtual QSize sizeHint() const
+    QSize sizeHint() const Q_DECL_OVERRIDE
     {
         const_cast<SideBarButton*>(this)->forceIcon();
         return QToolButton::sizeHint();
@@ -206,7 +206,7 @@ SideBar::SideBar(QWidget* parent)
 : QTabWidget(parent)
 , d(new SideBarPrivate)
 {
-    setFont(KGlobalSettings::smallestReadableFont());
+    setFont(QFontDatabase::systemFont(QFontDatabase::SmallestReadableFont));
     tabBar()->setDocumentMode(true);
     tabBar()->setUsesScrollButtons(false);
     tabBar()->setFocusPolicy(Qt::NoFocus);
