@@ -29,6 +29,7 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 // KDE
 #include <KDirLister>
 #include <KFileItem>
+#include <KProtocolManager>
 
 // Local
 #include <lib/document/documentfactory.h>
@@ -46,11 +47,13 @@ struct ContextManagerPrivate
     QUrl mCurrentUrl;
 
     QUrl mUrlToSelect;
+    QUrl mTargetUrl;
 
     bool mSelectedFileItemListNeedsUpdate;
     QSet<QByteArray> mQueuedSignals;
     KFileItemList mSelectedFileItemList;
 
+    bool mDirListerFinished = false;
     QTimer* mQueuedSignalsTimer;
 
     void queueSignal(const QByteArray& signal)
@@ -116,6 +119,8 @@ ContextManager::ContextManager(SortedDirModel* dirModel, QObject* parent)
     connect(d->mDirModel->dirLister(), SIGNAL(redirection(QUrl)),
             SLOT(slotDirListerRedirection(QUrl)));
 
+    connect(d->mDirModel->dirLister(), static_cast<void (KDirLister::*)()>(&KDirLister::completed), this, &ContextManager::slotDirListerCompleted);
+
     d->mSelectionModel = new QItemSelectionModel(d->mDirModel);
 
     connect(d->mSelectionModel, &QItemSelectionModel::selectionChanged, this, &ContextManager::slotSelectionChanged);
@@ -148,6 +153,7 @@ void ContextManager::setCurrentUrl(const QUrl &currentUrl)
         undoGroup->setActiveStack(doc->undoStack());
     }
 
+    d->mSelectedFileItemListNeedsUpdate = true;
     currentUrlChanged(currentUrl);
 }
 
@@ -162,9 +168,14 @@ void ContextManager::setCurrentDirUrl(const QUrl &url)
     if (url == d->mCurrentDirUrl) {
         return;
     }
-    d->mCurrentDirUrl = url;
-    if (url.isValid()) {
+
+    if (url.isValid() && KProtocolManager::supportsListing(url)) {
+        d->mCurrentDirUrl = url;
         d->mDirModel->dirLister()->openUrl(url);
+        d->mDirListerFinished = false;
+    } else {
+        d->mCurrentDirUrl.clear();
+        d->mDirModel->dirLister()->clear();
     }
     currentDirUrlChanged(url);
 }
@@ -275,10 +286,22 @@ void ContextManager::setUrlToSelect(const QUrl &url)
 {
     GV_RETURN_IF_FAIL(url.isValid());
     d->mUrlToSelect = url;
+    setCurrentDirUrl(url.adjusted(QUrl::RemoveFilename));
     setCurrentUrl(url);
     selectUrlToSelect();
 }
 
+QUrl ContextManager::targetUrl() const
+{
+    return d->mTargetUrl;
+}
+
+void ContextManager::setTargetUrl(const QUrl &url)
+{
+    GV_RETURN_IF_FAIL(url.isValid());
+    d->mTargetUrl = url;
+}
+
 void ContextManager::slotRowsInserted()
 {
     // We reach this method when rows have been inserted in the model, but views
@@ -295,11 +318,24 @@ void ContextManager::slotRowsInserted()
 
 void ContextManager::selectUrlToSelect()
 {
+    // Because of the queued connection above we might be called several times in a row
+    // In this case we don't want the warning below
+    if (d->mUrlToSelect.isEmpty()) {
+        return;
+    }
+
     GV_RETURN_IF_FAIL(d->mUrlToSelect.isValid());
     QModelIndex index = d->mDirModel->indexForUrl(d->mUrlToSelect);
     if (index.isValid()) {
         d->mSelectionModel->setCurrentIndex(index, QItemSelectionModel::ClearAndSelect);
         d->mUrlToSelect = QUrl();
+    } else if (d->mDirListerFinished) {
+        // Desired URL cannot be found in the directory
+        // Clear the selection to avoid dragging any local files into context
+        // and manually set current URL
+        d->mSelectionModel->clearSelection();
+        setCurrentUrl(d->mUrlToSelect);
+        d->mUrlToSelect.clear();
     }
 }
 
@@ -308,5 +344,10 @@ void ContextManager::slotDirListerRedirection(const QUrl &newUrl)
     setCurrentDirUrl(newUrl);
 }
 
+void ContextManager::slotDirListerCompleted()
+{
+    d->mDirListerFinished = true;
+}
+
 
 } // namespace
